#!/usr/bin/perl -Uw
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# activateLinuxUser.pl activates a disabled linux user if they are in the CSMuser group.
#
# Bernd Horn <bhorn@us.ibm.com> (Sept 5, 2000)
#

################################
# Let's use our needed modules #
################################

use User::grent;
use ASM::Exec;

#############################################
# First we have our configuration variables #
#############################################

use ASM::Common;
commonInit( ["userid=s"],	# unique options for this ocmmand
	    "--userid=CSMUSERID ", # unique usage message for this command
	    );

setFacility('sysuser');

if(!$opt_userid) {
  logUsage();
  exit(3);
}
else {

  my $pw = getpwnam $opt_userid;

# Check to see if user exists.

  if (!$pw) {
    error(MSG_NO_SUCH_USER, $opt_userid);
    exit (3);
  }

# Check to see if user is in CSMuser group.

  my $gr = getgr($CSMgrp);
  if ($pw->gid != $gr->gid) {
    error(MSG_NOT_ASM_USER, $opt_userid);
    exit 3;
  }

# Reactivate user
  my ($stdout, $stderr);
  my $rc = runit(["usermod", "-U", $opt_userid], \$stdout, \$stderr);
  if ($rc == 0) {
    audit(MSG_USER_ACTIVATED, $opt_userid);
  } else {
    trace("usermod -U failed:\n" . $stdout . $stderr);
  }
  exit $rc != 0;
}
