#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# addAllowedHosts.pl  adds the hosts allowed to access CSM
#
# Chi D Nguyen <cdn@us.ibm.com> (21 October 1999)
#

################################
# Let's use our needed modules #
################################


use ASM::Common;
use ASM::Log;
use ASM::Exec;

@opts = ("hostnames=s","admin=s", "help");  
$usage = "--hostnames=hostname1 [hostname n] [-admin=name] | --help";
commonInit(\@opts, $usage);
setFacility('syscfg');

my ($rc,$stdout,$stderr);

#############################################
# First we have our configuration variables #
#############################################

$CSMconfFile = "/opt/CSM/WebServer/conf/httpd.conf";
$CSMHostAllowSearch = "# CSM ALLOW HOSTS TO FOLLOW. DO NOT DELETE.";

# Get command line options and check 'em out to see if they're okay.

if ($Getopt::Long::error){  # There was an error in parsing the arguments
    logUsage();
    exit 3;
}

if($opt_help) {
  print STDERR "Usage: listAllowedHosts.pl $usage\n";
}
else {

  $tmpfilename = $CSMconfFile.`date -u +%T_%j`;
  chop $tmpfilename;
  $found = 0;
  if (! $opt_hostnames) {
      $opt_hostnames = "all";
  }
  else {
      &checkHostNames($opt_hostnames);
  }
  openOrDie(CSMCONFTEMP, ">$tmpfilename");
  openOrDie(CSMCONF, $CSMconfFile);
  while ($lines = <CSMCONF>) {
      print CSMCONFTEMP $lines;
      if ($lines =~ /$CSMHostAllowSearch/) {
	  print CSMCONFTEMP "    Allow from $opt_hostnames 127.0.0.1\n";
	  $linesAway = 0;
	  $lines = <CSMCONF>;
	  $found = 1;
      }
  }
  while ($lines = <CSMCONF>) {
      print CSMCONFTEMP $lines;
  }
  close(CSMCONF);
  close(CSMCONFTEMP);
  if ($found) {
      renameOrDie($tmpfilename, $CSMconfFile);
      audit(MSG_SET_ASM_ACCESS, $opt_hostnames);
      $rc = runit(["/etc/rc.d/init.d/asmhttpd", "graceful"], \$stdout, \$stderr);
      if ($rc) {
	  trace("asmhttpd graceful failed: $!\n$stdout\n$stderr");
      }
      exit $rc;
  }
  trace "The CSMhttpd.conf file is corrupted. \n";
  error(MSG_ERROR_SETTING_ASM_ACCESS);
  unlink($tmpfilename);
  exit 3;
}

sub checkHostNames {
    my $hostname = shift(@_);
    my @hostnames = split " ",$hostname;

    foreach my $host (@hostnames) {
      next if $host eq 'all';
      goodHostnameOrIPOrDie($host);
    }
}
