#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

use ASM::Common;
commonInit(["list"],
	   " [--list] ");

use strict;
BEGIN { $^W = 1; }

use POSIX qw|strftime|;
use IO::File;

use ASM::Log;
use ASM::MessageCodes;
use ASM::Exec;

POSIX::setlocale(&POSIX::LC_ALL,"");
setFacility("backcfg");

trace("enter backup.pl");
  
# Declare vars initialized in csm_common and used here
use vars qw/
  $auditcmd
  $backup_filename
  $backup_config_pattern
  $prebackup_script_pattern
  $opt_list
  /;

my ($rc, $stdout, $stderr);

if ($opt_list) {
  audit(MSG_LIST_BACKUPS);
  
  # If there's a backup, read its volume name
  # Otherwise, no output
  if (-f $backup_filename) {
    $rc = runit(["/bin/sh", "-c", "tar tf $backup_filename | head -1"], \$stdout, \$stderr);
    if ($rc != 0) {
      trace("Error $rc reading volume name from backup: " . $stdout . $stderr);
      exit(3);
    }
    my $name = $stdout;
    chomp($name);
    print $name;
    trace("backup name is $name");
    exit(0);
  } else {
    trace("No previous backup found ($backup_filename).");
  }
  exit(1);			# let caller know there were no backups
}

my $hostname = `hostname`;
chomp($hostname);
#my $volume_name = strftime("$hostname %c", localtime());
my $volume_name = strftime("$hostname %Y-%m-%d  %T", localtime());

audit(MSG_E77);		# backup started

# Run any pre-backup scripts
my @scripts = glob($prebackup_script_pattern);
foreach my $script (@scripts) {
  if (-x $script) {
    trace("running $script");
    $rc = runit([$script], \$stdout, \$stderr);
  }
  else {
    trace("$script is not executable");
  }
}

# Look for config files
# We want to use -T to feed filenames to tar, but it appears to ignore
# all but the last occurrence.  So we need to build a temp file and
# put all our filenames tnere.
my @config_files = glob($backup_config_pattern);
my $config_arg = "";

my $out = new IO::File ">/tmp/backup.filelist";
if (!$out) {
  trace("Could not create /tmp/backup.filelist: $!\n");
  error(MSG_COULD_NOT_OPEN_FILE, "/tmp/backup.filelist");
  exit(3);
}
foreach (@config_files) {
  trace("Reading $_...");
  my $in = new IO::File "<$_";
  if (!$in) {
    trace("Error opening $_: $!\n");
    error(MSG_COULD_NOT_OPEN_FILE, $_);
    exit(3);
  }
  print $out join('',<$in>);
  close($in);
}
close($out);

$rc = runit(["tar -Pcvpf ${backup_filename}-new -V \"$volume_name\" --ignore-failed-read -T /tmp/backup.filelist"], \$stdout, \$stderr);
if ($rc != 0) {
  trace("Error while tar backup file: " . $stdout . $stderr);
  error(MSG_BACKUP_FAILED);
  exit(3);
}
# Okay, backup succeeded
if (-f $backup_filename) {
  unlink($backup_filename);
}
rename("${backup_filename}-new", $backup_filename)
  or do {
    trace("Error while renaming ${backup_filename}-new $backup_filename: $!");
    error(MSG_BACKUP_FAILED);
    exit(3);
  };

audit(MSG_E50);
  
exit(0);
