#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#



# changeUserQuota.pl creates/modifies the quota of a linux user.
#
# Pierre Carlson <mpcarl@us.ibm.com> (12 October 1999)
#
# Script called to query or set a users quota.  Sets the EDITOR environment
# variable to /opt/CSM/scripts/edQuota.pl and the calles the system command
# edquota.  edquota will then call our edQuota.pl script to do the editing.  
# The value is passed via the environment variable USER_HARD_LIMIT.

################################
# Let's use our needed modules #
################################

use DB_File;
use Fcntl;
use User::pwent;

#############################################
# First we have our configuration variables #
#############################################

use ASM::Common;
commonInit(["userid=s", "disklimit=i", "query"],
	   "--userid CSMUSERID [ --disklimit HARDLIMIT | --query ] ");
$device = "";

setFacility('sysuser');

if(!$opt_userid) {
  logUsage();
  exit(3);
} 
elsif( defined ($opt_disklimit) && $opt_query) {
  logUsage();
  exit(3);
}
elsif($opt_query == 1) {
    &getPartition();
    &checkReqs();
    &doQuery();
    exit 0;
}
elsif(defined ($opt_disklimit)) {
    &getPartition();
    &checkReqs();
    &doEdit();
    exit 0;
} else {
  logUsage();
  exit(3);
}

# sub routine to check if the prereqs are met.
sub checkReqs {
# Make sure the user already exists
    $pw = getpwnam $opt_userid;
    if (!$pw) {
      error(MSG_NO_SUCH_USER, $opt_userid);
      exit (4);
    }
}
#sub routine to check query the current setting
# write the current value to STDOUT
sub doQuery {
    $tmpFile = "/tmp/$opt_userid.quota";
    $rc = runit(["rm -f $tmpFile"], \$stdout, \$stderr);
    $ENV{"EDITOR"} = "cat";
    $curLimit = 0;
    $rc = runit(["/usr/sbin/edquota -u $opt_userid > $tmpFile"], \$stdout, \$stderr);
    openOrDie(INFILE, "< $tmpFile");
    while (defined ($line = <INFILE>)) {
	if ($line =~ m|^$device|) {
	    @AR = split /\s+/, $line ;
	    chop($AR[11]);
	    $curLimit = $AR[11];
	} 
    } # end while
    
    print STDOUT "$curLimit\n";

    close(INFILE);
    $rc = runit(["rm -f $tmpFile"], \$stdout, \$stderr);
}

# sub routine to set a new limit
# write the new value to STDOUT
sub doEdit {
# set the EDITOR Environment variable so edquota will call it instead of vi.
    $ENV{"EDITOR"} = "/opt/CSM/scripts/edQuota.pl";
    $ENV{"USER_HARD_LIMIT"} = "$opt_disklimit";
    $rc = runit(["/usr/sbin/edquota -u $opt_userid"], \$stdout, \$stderr);
    audit(MSG_USER_QUOTA_CHANGE, $opt_userid, $opt_disklimit);
}

# this routine will set the users home directory partition
# in the env variable "USER_PARTITION"
sub getPartition {

    my $topDir = "";

    checkReqs();		# dies unless user exists

    my $pw = getpwnam($opt_userid);
    # $pw->dir nw contais the home directory 

    my $home = $pw->dir;
    openOrDie(FIN, "df $home |");
    while(<FIN>)
    {
      if (m/^(\S+)/)
      {
	$device = $1;
      }
    }
    close(FIN);

    $ENV{"USER_QUOTA_DEVICE"} = "$device";

    return;

   
#    if ($pw->dir =~ m|^(/home2?).*|) {
#	$topDir = $1;
#    }
#    else {
#	$topDir = '/';
#    }

    # get the partition name fro /etc/fstab
#    my($fstabFile)="/etc/fstab";
#
#    $H=tie(@h,'DB_File',$fstabFile,O_RDWR,0644,$DB_RECNO)
#	or die "can't update /tmp/sampe: $!";
#    for ($i=0; $i<$H->length; ++$i) {
#	$_=$h[$i];
#	if(/$topDir/i){
#	    $thisEntry = $_;
#	    #$i = $H->length; # break
#	    last;
#	}
#    }    
#    untie @H;
#
#    ($device, $home) = split(/\s+/,$thisEntry); 
#     $ENV{"USER_QUOTA_DEVICE"} = "$device";

}
