#
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#


#
# common_install.pl - cfg variable and common functions used by the Install/Uninstall CSM scripts.
#

$IMG_DIR = "/images/install";
$IMG_TMP = "/images/tmp";

$BK_FS = "/backup";
$BK_DIR = "/backup/installed";  #Formerly $bkup_home 
$UPDATES_FILE = "/opt/CSM/data/updates";
$STATUS_FILE = "/images/getImage.status";    # status of getImage.pl script.
$MNT_PT = "/mnt/cdrom";

use Cwd;
use IO::Socket;
use Net::FTP;
use ASM::Common;

sub addUpdate
{
   my ($ptfname) = @_;

   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
   # Try to format the timestamp fairly culturally neutral...
   my $timestamp = sprintf("%04d-%02d-%02d.%02d:%02d:%02d", $year + 1900, $mon+1, $mday, $hour, $min, $sec);

   openOrDie(UPD, ">> $UPDATES_FILE");
   print UPD "$ptfname $timestamp\n";
   close UPD;
   return(1); 
}

sub displaygetImageStatus
{
   my (@line, @tok);
   my ($rc, $stdout, $stderr);

   #  For one reason or another, status file was not created.
   #  Currently, workaround where /images is mounted after
   #  ASM is already installed.
   if (! -e "$STATUS_FILE")
   {
      $rc = runit(["> $STATUS_FILE"], \$stdout, \$stderr);
   }
   openOrDie(STA, "< $STATUS_FILE");
   #  Should only be 1 status entry but loop in case.
   while (<STA>)
   {
      print;
   }
   close STA;
   return(1);
}

sub displayUpdates
{

   openOrDie(UPD, "< $UPDATES_FILE");
   while (<UPD>)
   {
      print;
   }
   close UPD;
   return(1);
}

sub isURLreachable
{
   my ($ftpurl) = @_;
   my ($protocol, $server, $dir, $port, $rc, $stdout, $stderr);

   ($protocol, $server, $dir, $port) = &parseURL($ftpurl);
   $rc = runit(["/bin/ping -c1 $server"], \$stdout, \$stderr);
   if ($rc != 0)
   {
      trace("Output of ping command: " . $stdout . $stderr);
      trace("ERROR: ping comamnd failed.");
      return (0);
   }
   else
   {
      return(1);
   }
}

sub parseURL
{
   my ($input) = @_;
   my ($url, $protocol, $remote_host, $path, $port, $hostname) = "";

   # Next, split off the protocol. If a protocol is not given, default it to ftp
   if ($input =~ /:\/\//)
   {
     ($protocol, $url) = split(/:\/\//, $input);
   }
   else
   {
      #  Default to ftp.
      $protocol = "ftp";
      $url = $input;
   }

   # The very first component of the URL will be the remote host name.
   # Only one split with respect to '/' is desired.
   ($remote_host, $url) = split(/\//, $url, 2);
   
   # get the port, if one is specified
   if ($remote_host =~ /:/)
   {
      ($remote_host, $port) = split(/:/,$remote_host);
   }
   
   # if there is no hostname, use the locahost    (ERROR???)
   unless ($remote_host)
   {
      chomp($hostname=`hostname`);
      $remote_host = $hostname;
      trace("No hostname in URL.");
   }
   
   # not all systems know what the port number of ftp is, so
   # if it doesn't, we hardwire the value.
   unless ($port)
   {
      if ($protocol == "ftp")
      {
         $port = 21;
      }
      else
      {
         #  Unsupported protocol
         trace("Unknown/unsupported protocol" . $protocol . " specified.");
      }
   }

   $path = $url;

   return($protocol, $remote_host, $path, $port);
}

sub mountit
{
   my ($type, $server, $exportdir) = @_;
   my ($rc, $stdout, $stderr);

   $rc = 0;

   if ($type eq "cd")
   {
      trace("+++ mount the CD on $MNT_PT +++");
      $rc = runit(["mount /dev/cdrom $MNT_PT -t iso9660 -o noauto,ro,user,nohide"], \$stdout, \$stderr);
      #  Sometimes the filesystem is already mounted (previous  umount fails, etc.)
      if ($rc != 0)
      {
         &umountit();
         $rc = runit(["mount /dev/cdrom $MNT_PT -t iso9660 -o noauto,ro,user,nohide"], \$stdout, \$stderr);
      }
      if ($rc != 0)
      {
         trace("Output of command: $stdout $stderr");
         trace("Error: Failed to mount the $type filesystem with rc: $rc.");
         $rc = 71;
         return($rc);
      }
   }
   else
   {
      if ($server eq "")
      {
	 trace("Error: Server name needed for file transfer.");
         $rc = 57;
         return($rc);
      }
      if ($exportdir eq "")
      {
	 trace("Error: export directory needed for file transfer.");
         $rc = 70;
         return($rc);
      }
      trace("+++ nfs mount at $MNT_PT +++");
      #
      # Mount filesystem
      #
      trace("Running command: mount $MNT_PT");
      $rc = runit(["/bin/mount $server:$exportdir $MNT_PT -t nfs -o noauto,rw,user,rsize=8192,wsize=8192"], \$stdout, \$stderr);
      if ($rc != 0)
      {
         trace("Output of command: $stdout $stderr");
         trace("Error: Failed to mount the $type filesystem with rc: $rc.");
         $rc = 71;
         return($rc);
      }
   }
  
   trace("Successfully mounted on $MNT_PT");
  
   return($rc);
}

sub umountit
{
   my ($rc);

   $rc = 0;

   trace("Mount value: $mounted   $MNT_PT"); 
   trace("Attempting umount of $MNT_PT");
   $cnt = 5;
   #  Be sure to be in a neutral directory before trying to umount.
   $rc = chdir("/");
   while ($cnt > 0)
   {
      $rc = runit(["/bin/umount -f $MNT_PT"], \$stdout, \$stderr);
      if ($rc == 0)
      {
        last;
      }
      sleep(1);
      $cnt--;
   }
   if ($rc != 0)
   {
      trace("Output of command: $stdout $stderr");
      my $currdir = cwd();
      trace("Couldn't unmount the mount point: $MNT_PT.  CWD = $currdir.  Output of umount: $stdout $stderr");
   }
   return($rc);
}

sub getPTFList
{
   my ($type, $ftpurl, $ver, $rel, $ptf) = @_;
   my (@ptflist, @dirs, $ptfint, $currdir, @files, $currfile, $ptfversion, $ptfrelease, $ptflevel, $ptflevelint);
   my ($protocol, $server, $path, $port);
   my ($currpath);
   my ($debug) = 0;

   #   Search directories on remote ftp server.
   #   For upgrade detection, it is assumed that the
   #   PTF fixes are in the first level subdirectories.
   #   For eaxample:
   #         ibmasmWebHost
   #                  ibmasmRelease-1.2-1.pkgInfo.txt
   #                  ibmasmRelease-1.2-1.tar.gz
   #         ibmEdgeServer
   #                  ibmasmRelease-2.2-1.pkgInfo.txt
   #                  ibmasmRelease-2.2-1.tar.gz
   #

   @ptflist = ();

   if ($type eq "ftp")
   {
      print "FTPURL: $ftpurl\n" if $debug;

      #  Currently, only traversing down 1 directory level.
      @dirs = &remotedirlist($ftpurl, "dir");
   }
   else
   {
      print "FILESYSTEM: $ftpurl\n" if $debug;

      #  Currently, only traversing down 1 directory level.
      @dirs = &dirlist($ftpurl, "dir");
   }
   if ($ptf =~ /[0-9]*/)
   {
      $ptfint = $ptf - 0;
   }
   else
   {
      trace("Error:  Invalid PTF level $ptf specified.");
      return(@ptflist);
   }

   if ($type eq "ftp")
   {
      #  Also look for files in the given url directory
      ($protocol, $server, $path, $port) = &parseURL($ftpurl);
      chomp($path);
   }
   else
   {
      $path = $ftpurl;
   }

   if ($path eq "")
   {
      $path = "/";
   }

   print "Adding $path to list of directories.\n" if $debug;
   push(@dirs, ".");

   foreach $currdir (@dirs)
   {
      if ($type eq "ftp")
      {
         @files = &remotedirlist("$ftpurl/$currdir", "file");
      }
      else
      {
         @files = &dirlist("$ftpurl/$currdir", "file");
      }
      if ($path eq "/")
      {
         $currpath = $currdir;
      }
      else
      {
         if ($currdir eq ".")
         {
            $currpath = $path;
         }
         else
         {
            $currpath = "$path/$currdir";
         }
      }
      print "Looking for files in directory: $currpath\n" if $debug;
      foreach $currfile (@files)
      {
         $_ = $currfile;
         print "Checking file: $currfile\n" if $debug;
         ($ptfversion, $ptfrelease, $ptflevel) = $currfile =~ /^ibmasmRelease\-([0-9]*)\.([0-9]*)-([0-9]*)\..*/;
         chomp($ptfversion);
         chomp($ptfrelease);
         chomp($ptflevel);
         if (($ptfversion eq $ver) && ($ptfrelease eq $rel))
         {
            print "Found ptflevel: $ptflevel\n" if $debug;
            if ($ptflevel =~ /[0-9]*/)
            {
               $ptflevelint = $ptflevel - 0;
               chomp($ptflist[$ptflevelint]);
               if (!$ptflist[$ptflevelint] || $ptflist[$ptflevelint] eq "")
               {
                  if ($currpath eq "/")
                  {
                     #  File located at top level directory
                     $ptflist[$ptflevelint] = "ibmasmRelease-$ver\.$rel-$ptflevel / /$currfile";
                  }
                  else
                  {
                     $ptflist[$ptflevelint] = "ibmasmRelease-$ver\.$rel-$ptflevel $currpath $currpath/$currfile";
                  }
               }
               else
               {
                  #  Validate what's there?  Files should only be ibmasmRelease-xx.yy-zz.pkgInfo.txt and ibmasmRelease-xx.yy-zz.tar.gz
                  #  Previous ptf file was found.  Add this one to it.  Order of the ptf files doesn't matter.
                  if ($currpath eq "/")
                  {
                     #  File located at top level directory
                     $ptflist[$ptflevelint] = $ptflist[$ptflevelint] . " /$currfile";
                  }
                  else
                  {
                     $ptflist[$ptflevelint] = $ptflist[$ptflevelint] . " $currpath/$currfile";
                  }
               }
            }
            else
            {
               #  Not a valid PTF file, Skip it.
               trace("Warning:  Invalid PTF file $currfile in $currpath.  Skipping it.");
            }
         }
         else
         {
            #  Not a valid PTF file, Skip it.
            trace("Warning:  Invalid PTF file $currfile found in $currpath.  Skipping it.");
         }
      }
      #  At this point, some ptfs may be missing in the array.
   }
   return(@ptflist);
}

#  Anonymous remote dir list (http or ftp)
sub remotedirlist
{     	# begin subroutine
   # Get input arguments and initialize variables.
   my ($input, $type) = @_;
   my ($error, $port, $hostname, $xf, $oldout, $protocol, $url, $remote_host);
   my ($sockaddr, $name, $aliases, $proto, $len, $thisaddr, $this, $that, $url_request);
   my ($thataddr, $status, @output, $query_string);
   my ($savehand);
   my ($ftp, @list, $dir, @dir_list);
   my ($debug) = 0;
   
   $port = "";
   chomp($hostname=`hostname`);
   $error = 0;
   @dir_list = ();
   
   # Next, split off the protocol. If a protocol is not given, default it to ftp
   if ($input =~ /:\/\//)
   {
     ($protocol, $url) = split(/:\/\//, $input);
   }
   else
   {
      $protocol = "ftp";
      $url = $input;
   }

   #  CUrrently only support FTP
   if ($protocol ne "ftp")
   {
      #  Unsupported protocol
      trace("Unknown/unsupported protocol" . $protocol . " specified.");
      return(@dir_list);
   }

   # The very first component of the URL will be the remote host name.
   # Only one split with respect to '/' is desired.
   ($remote_host, $url) = split(/\//, $url, 2);
   
   # get the port, if one is specified
   if ($remote_host =~ /:/)
   {
      ($remote_host, $port) = split(/:/,$remote_host);
   }
   
   # if there is no hostname, use the locahost    (ERROR???)
   unless ($remote_host)
   {
      $remote_host = $hostname;
      trace("No hostname in URL.   Defaulting to localhost.");
   }
   
   # not all systems know what the port number of ftp is, so
   # if it doesn't, we hardwire the value.
   unless ($port)
   {
      if ($protocol == "ftp")
      {
         $port = 21;
      }
      else
      {
         #  Unsupported protocol
         trace("Unknown/unsupported protocol" . $protocol . " specified.");
         return(@dir_list);
      }
   }

   $ftp = Net::FTP->new($remote_host, Passive => 1);
   if ($ftp eq undef || !$ftp)
   {
      error(MSG_UNABLE_TO_CONNECT, $remote_host);
      trace("Unable to connect to $remote_host: $@");
      return(@dir_list);
   }
   if (!$ftp->login())
   {
      error(MSG_UNABLE_TO_LOGIN, $remote_host, "anonymous");
      trace("Unable to log into the remote FTP server using userid: anonymous");
      return(@dir_list);
   }
   $ftp->binary();

   $url_request = "$url";

   print "Looking for ${type}s in directory: $url_request\n" if $debug;

   $ftp->cwd($url_request);
   my ($currdir);
   $currdir = $ftp->pwd();
   print "Current working directory: $currdir\n" if $debug;

   @list = $ftp->dir();
   #  Parse out the filename/directories in the long listing output
   @list = &parselist(@list);

   print "Checking to see if files/directories are type $type: @list\n" if $debug;
   foreach $dir (@list)
   {
      $ftp->cwd($url_request);
      my ($currdir);
      $currdir = $ftp->pwd();
      print "Current working directory: $currdir\n" if $debug;
      print "Checking $type: $dir\n" if $debug;
      if ($dir ne "." && $dir ne "..")
      {
         if ($ftp->cwd("$dir"))
         {
            #  Found a directory.
            if ($type eq "dir")
            {
               push(@dir_list, $dir);
            }
         }
         else
         {
            #  Found a File.
            if ($type ne "dir")
            {
               push(@dir_list, $dir);
            }
         }
      }
   }
   
   $ftp->quit; 
   return(@dir_list);

} # end extract function

sub parselist
{
   my (@currlist) = @_;
   my ($newlist, @fields, $line);

#  The dir command produces a long listing of a dir.
#  The last field on each line should be a filename or directory name.
#  The first line of the output will be a total number of files/directories
#  and will be ignored.

   $newlist = "";

   shift(@currlist);
   foreach $line (@currlist)
   {
      #  Assuming last field is the dir/file.
      @fields = split(/\s+/, $line);
      $newlist .= $fields[$#fields]. " ";
   }

   return(split(/\s+/,$newlist));
}

#  Filesystem dir list (CD, NFS, or Local)
sub dirlist
{     	# begin subroutine
   # Get input arguments and initialize variables.
   my ($path, $type) = @_;
   my ($currdir, $error, @list, $dir, @dir_list);
   my ($debug) = 0;
   
   $error = 0;
   @dir_list = ();
   
   my $savedir = cwd();

   print "Looking for ${type}s in directory: $path\n" if $debug;

   $rc = chdir($path);
   my $currdir = cwd($path);
   print "Current working directory: $currdir\n" if $debug;
   if (!opendir(CURRDIR, $path))
   {
      error(MSG_COULD_NOT_OPEN_DIRECTORY, $path);
      $rc = chdir($savedir);
      return(@dir_list);
   }
   @list = readdir(CURRDIR);
   closedir(CURRDIR);
   #@list = split(`ls -1  $path`);
   print "Checking to see if files/directories are type $type: @list\n" if $debug;
   foreach $dir (@list)
   {
      $rc = chdir($path);
      my $currdir = cwd($path);
      print "Current working directory: $currdir\n" if $debug;
      print "Checking $type: $dir\n" if $debug;
      if ($dir ne "." && $dir ne "..")
      {
         if (chdir("$dir"))
         {
            #  Found a directory.
            if ($type eq "dir")
            {
               push(@dir_list, $dir);
            }
         }
         else
         {
            #  Found a File.
            if ($type ne "dir")
            {
               push(@dir_list, $dir);
            }
         }
      }
   }
   
   $rc = chdir($savedir);
   return(@dir_list);

} # end extract function
sub findVersion
{
   my ($ver_data, $version, $release, $ptf);

   $ver_data = `cat /opt/CSM/release`;
   chomp($ver_data);
   ($version, $release, $ptf) = $ver_data =~ /(.*)\.(.*)-(.*)/;

   return($version, $release, $ptf);
}

1;
