#!/usr/bin/perl 
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# configureMailmgr    -  Configure and Enable the Mail List Manager to sendmail
#                       [--init]
#                       [--adminaddr=<administrator's email address>]
#
# Configures/Activates Mail List Mgr

use ASM::Exec;
use ASM::Common;

my ($eSP_BIN, $rc, $stdout, $stderr, @opts, $configureMailmgr_usage, $majordomoaliases);
my ($securewrapper, $wrapper, $mjadmin, $sendmail_cf, $sendmail_mc, $maillist_aliases, $hostname);

$eSP_BIN = "/opt/CSM/scripts";

require "$eSP_BIN/common_mailmgr.pl";

use strict;

@opts = ("adminaddr=s", "init");
$configureMailmgr_usage = "[--addr=<administrator's email address>] [--init]";
commonInit(\@opts, $configureMailmgr_usage);

setFacility('sysuser');

use vars qw/ $MAILMGR_CF $MAILMGR_ALIASES $MAILMGR_WRAPPER $SECURE_WRAPPER $opt_help $opt_adminaddr $opt_init $log /;
require "$MAILMGR_CF";

audit(MSG_CONFIG_MAILMGR_STARTED);

$maillist_aliases = "$MAILMGR_ALIASES";
$wrapper = "$MAILMGR_WRAPPER";
$securewrapper = "$SECURE_WRAPPER";
$mjadmin = "root";
$sendmail_cf = "/etc/sendmail.cf";
$sendmail_mc = "/etc/sendmail.mc";

# Get command line options and check 'em out to see if they're okay.
&GetOptions(@opts);

if ($Getopt::Long::error){  # There was an error in parsing the arguments
  exit 2;
}

if ($opt_help)  {
  printUsage();
}

if ($opt_init)
{
   &create_alias_maillist($mjadmin);
   #  Correct the Mail List Config file for the hostname
   runit(["/bin/cp -f $MAILMGR_CF $MAILMGR_CF.prev"], \$stdout, \$stderr);
   $rc = runit(["/bin/sed -e \'s/dnsdomainname/hostname/\' < $MAILMGR_CF > $MAILMGR_CF.new"], \$stdout, \$stderr);
   if ($rc != 0)
   {
      trace("Error modifying Mail List Manager $MAILMGR_CF config file: $stdout $stderr\n");
      error(MSG_CONFIG_MAILMGR_FAILED);
      exit($rc);
   }
   $rc = runit(["cat $MAILMGR_CF.new > $MAILMGR_CF"], \$stdout, \$stderr);
   if ($rc != 0)
   {
      trace("Error writing new Mail List Manager $MAILMGR_CF config file: $stdout $stderr\n");
      error(MSG_CONFIG_MAILMGR_FAILED);
      exit($rc);
   }

   runit(["rm -f $MAILMGR_CF.new"], \$stdout, \$stderr);

}

if ($opt_adminaddr)
{
   #  Check for valid email address?
   $mjadmin = $opt_adminaddr;
   &set_admin_maillist($mjadmin);
}

# Build the M4 input file for the sendmail configuration
# The base configuration is based on RedHat's

  # Start with /etc/sendmail.mc
  # Replace any AliasFile with one with MailList Alias file

my @config;
push(@config, "# Sendmail configuration created by $0.\n");

openOrDie(IN, "<$sendmail_mc");
while (<IN>)
{
   if (/ALIAS_FILE/)
   {
      next;			# skip our own comment, SMART_HOST, and ALIAS_FILE
   }
   push(@config, $_);
}
close(IN);

push(@config, "define(`ALIAS_FILE', `/etc/aliases,$maillist_aliases')\n");

openOrDie(OUT, ">$sendmail_mc");
print OUT @config;
close(OUT);

# "Compile" it
$rc = runit(["/usr/bin/m4", $sendmail_mc], \$stdout, \$stderr);
if ($rc != 0) {
  error(MSG_SENDMAIL_UPDATE_ERROR);
  trace("Errors from m4: $stderr\n");
  exit(1);
}

# Backup config if not previously done (at least for development
# so I don't have to completely re-install to fix myself...)
if (! -f "${sendmail_cf}-backup") {
  system("cp $sendmail_cf ${sendmail_cf}-backup");
}

# Replace config
openOrDie(OUT, ">$sendmail_cf");
print OUT $stdout;
close(OUT);

# Restart sendmail
$rc = runit(["/etc/rc.d/init.d/sendmail", "restart"], \$stdout, \$stderr);
if ($rc != 0) {
  error(MSG_SENDMAIL_UPDATE_ERROR);
  trace("Errors restarting sendmail: $stdout $stderr\n");
}

#  Create Mailmgr log file.
$rc = runit(["/bin/touch $log"], \$stdout, \$stderr);
if ($rc != 0)
{
  #  Continue if this fails since its a minor problem. 
  trace("Error creating Mail List Manager log file: $stdout $stderr\n");
}
else
{
   runit(["/bin/chown majordomo $log"], \$stdout, \$stderr);
   runit(["/bin/chgrp majordomo $log"], \$stdout, \$stderr);
   runit(["/bin/chmod 0664 $log"], \$stdout, \$stderr);
}

#  Add wrapper to sendmail's secure program area
$rc = runit(["/bin/ln -f -s $wrapper $securewrapper"], \$stdout, \$stderr);
if ($rc != 0) {
  error(MSG_CONFIG_MAILMGR_FAILED);
  trace("Error adding wrapper to secure sendmail programs: $stdout $stderr\n");
  exit($rc);
}

audit(MSG_CONFIG_MAILMGR_COMPLETED);
exit(0);
