#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# ----------------------------------------------------------------------
# Perl script that knows how to invoke manageSSLCert.pl with all the
# right options to create a self-signed certificate for our admin
# server to use
# 
# Dan Poirier, 16 Aug 2000
# ----------------------------------------------------------------------

# Modules to use
use Getopt::Long;
use ASM::Common;
use ASM::Exec;
commonInit([],"");

use strict;
BEGIN { $^W = 1; }
use vars qw! $auditcmd !;

setFacility('security');
audit(MSG_CREATE_SELF_SIGNED_CERT);

my ($rc,$stdout,$stderr);

my $ipaddress;
open(DEVICE,"/opt/CSM/scripts/device.pl --get --all | " );
while (<DEVICE>) {
  if (/^eth0/) {
    (undef,$ipaddress,undef) = split(' ', $_);
  }
}
close(DEVICE);

my $directory = "/opt/CSM/WebServer/SSLCertificate";

my $keyfile = "$directory/key.kdb";
my $password = "ASMSSL";
my $certfile = "$directory/sscert.arm";
my $label = "ASM SelfSigned Key";
my $version = "X509V3";
my $organization = "IBM";
my $size = 1024;
my $country = "us";
my $valid_days = 3650;

# Save old key files if there, just in case
my @savefiles;
my $savefile = "/backup/keys.tar";
unlink($savefile);
for my $file ("key.kdb", "key.sth", "key.crl", "key.rdb") {
  if (-f "$directory/$file") {
    push(@savefiles, "$directory/$file");
  }
}
if (@savefiles) {
  $rc = runit(["tar -cPf $savefile " . join(" ", @savefiles)], \$stdout, \$stderr);
}

# Remove old files
unlink(@savefiles);

# Create new key database file
if (! -f $keyfile) {
  $rc = runit(["/opt/CSM/scripts/manageSSLCert.pl --create --KDB --keyfile $keyfile --password $password"],
	      \$stdout, \$stderr);
  if ($rc != 0) {
    trace("error creating ssl database file: rc=$rc\n$stdout\n$stderr");
    &recover();
    exit(12);
  }
}

# Create cert
$rc = runit(["/opt/CSM/scripts/manageSSLCert.pl --create --SelfCert "
	     . " --keyFile $keyfile --password \"$password\" --certFile $certfile --label $label"
	     . " --version $version --size $size --commonName $ipaddress "
	     . " --org $organization --country $country --valid $valid_days"],
	    \$stdout, \$stderr);
if ($rc != 0) {
  trace("error creating certificate: rc=$rc\n$stdout\n$stderr");
    &recover();
  exit(12);
}

# Clean up
unlink($savefile);

# Must restart Web server to make change take effect
# IHS restart doesn't work right now, use stop then start

runit(["/etc/rc.d/init.d/asmhttpd graceful"],\$stdout,\$stderr);
runit(["/etc/rc.d/init.d/ibmhttpdadmin graceful"],\$stdout,\$stderr);

exit(0);

sub recover {
# Try to restore previous key files
  if (-f $savefile) {
    runit(["tar xvf $savefile"],\$stdout,\$stderr);
    unlink($savefile);
  }
}
