#!/bin/bash
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
#
# deleteImage.pl - Clear /images/install directory
#  
# Wayne Gao, <waynegao@us.ibm.com> (27 Octobe 1999)
#
use ASM::Common;

use strict;
my ($eSP_BIN, @opts, $dir_name, $stdout, $stderr, $status);

$eSP_BIN = "/opt/CSM/scripts";

require "$eSP_BIN/common_install.pl";

commonInit([],"");

setFacility('upgrade');

use vars qw/ $opt_help $IMG_DIR /;

sub cleanup_exit
{
   my ($exitmsg, $exitrc) = @_;
 
   trace("$exitmsg");
   trace("exit $exitrc");

   if ($exitrc != 0)
   {
      error(MSG_UPDATE_DELETE_FAILED, $IMG_DIR);
   }
   else
   {
      audit(MSG_UPDATE_DELETE_COMPLETED, $IMG_DIR);
   }

   exit $exitrc;
}

sub sig_handler
{
   my ($sig) = @_;
   audit(MSG_CAUGHT_SIGNAL, $sig);
   &cleanup_exit("Signal $sig caught.   Exiting..", 1);
}

sub setup_signals
{
   $SIG{'TERM'} = 'sig_handler';
   $SIG{'QUIT'} = 'sig_handler';
   $SIG{'INT'} = 'sig_handler';
   $SIG{'HUP'} = 'sig_handler';

   return( 0 );
}

&setup_signals();

# Parse the arguments
@opts = ("admin=s", "help");
 
# Get command line options and check 'em out to see if they're okay.
 
&GetOptions(@opts);

if ($Getopt::Long::error)
{  # There was an error in parsing the arguments
   &cleanup_exit("Error:  Unknown problem with parsing arguments.", 2);
}

if($opt_help)
{
   &printUsage; 
}                                          


trace("====== `date|tr -d \"\n\"` ======");
chdir $IMG_DIR;
$dir_name = `pwd`;
if ($dir_name eq $IMG_DIR)
{
   trace("Clean up $IMG_DIR");
   trace("rm -rf $IMG_DIR/*");
   $status = runit(["rm -rf $IMG_DIR/*"], \$stdout, \$stderr);
   if ($status == 0)
   {
      &cleanup_exit("Successfully cleaned  $IMG_DIR", 0);
   }
   else
   {
      &cleanup_exit("Error: Failed to clean $IMG_DIR.  rm rc = $status", $status);
   }
}
else
{
  &cleanup_exit("Error: Couldn't chdir to $IMG_DIR.", 1);
}

&cleanup_exit("", 0);
