#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#


# deleteLinuxUser.pl deletes a linux user if they are in the CSMuser group.
#
# Chi D Nguyen <cdn@us.ibm.com> (5 October 1999)
#

use User::grent;
use ASM::Exec;

#############################################
# First we have our configuration variables #
#############################################

use ASM::Common;
commonInit(["userid=s","disable"],
	   "--userid=CSMUSERID [--disable]");

if(!$opt_userid) {
  logUsage();
  exit(3);
}
else {
  $pw = getpwnam $opt_userid;

  my ($rc, $stdout, $stderr);

# Check to see if user exists.

  if (!$pw) {
    error(MSG_NO_SUCH_USER, $opt_userid);
    exit (3);
  }

# Check to see if user is in CSMuser group.

  my $gr = getgr($CSMgrp);
  if ($pw->gid != $gr->gid) {
    error(MSG_NOT_ASM_USER, $opt_userid);
    exit 3;
  }

  if ($opt_disable) {
    audit(MSG_USER_DISABLED, $opt_userid);
    $rc = runit(["usermod -L $opt_userid"], \$stdout, \$stderr);
    exit $rc;
  }

  # Remove quota for user
  $rc = runit(["/opt/CSM/scripts/changeUserQuota.pl --userid $opt_userid --disklimit 0" . ($opt_admin ? " --admin $opt_admin" : "")], \$stdout, \$stderr);

# This is a workaround to remove the home directory,since userdel -r 
# does not work with userid = 99
	     
  if ($pw->dir =~ m@^/home@)
  {
      $rc = runit(["/bin/rm -rf " . $pw->dir], \$stdout, \$stderr);
  }

# Delete user.

  audit(MSG_USER_DELETED, $opt_userid);
  remove_old_lock();
  $rc = runit(["userdel $opt_userid"],\$stdout,\$stderr);
  remove_old_lock();

  # Remove any email alias
  $rc = runit(["/opt/CSM/scripts/setEmailAlias.pl", "--from=$opt_userid",
	       "--delete"], \$stdout, \$stderr);

  if ($rc != 0) {
    error(MSG_SENDMAIL_UPDATE_ERROR);
    trace("Errors changing alias: $stdout $stderr\n");
  }

  exit $rc;
}

sub remove_old_lock {
  # Work around apparent bug in useradd...
  if (-f "/etc/group.lock") {
    chomp(my $pid = `cat /etc/group.lock`);
    if (kill(0,$pid)) {
      # Pid still exists, don't remove
      #print "not removing apparently valid lock\n";
    } else {
      #print "REMOVED OLD LOCK!!!\n";
      unlink("/etc/group.lock");
    }
  }
}
