#!/usr/bin/perl 
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# displayMaillist.pl
#
# Shows information about a Mail List.
#
# Usage:
#   displayMailmgr.pl [--help]
#   
#   Shows the currently configured Mail List Mgr status (Enabled/Disabled)

use ASM::Exec;
use ASM::Common;

my (@eSP_BIN, $rc, $stdout, $stderr, $sendmail_mc, $enabled);
$eSP_BIN = "/opt/CSM/scripts";

require "$eSP_BIN/common_mailmgr.pl";

#  Parse the arguments
commonInit([], "");

setFacility('sysuser');

use vars qw/ $opt_help $opt_name /;

# Get command line options and check 'em out to see if they're okay.

&GetOptions(@opts);

if ($Getopt::Long::error){  # There was an error in parsing the arguments
  exit 2;
}

if ($opt_help)  {
  printUsage();
}

sub cleanup_exit
{
   my ($exitmsg, $exitrc) = @_;

   print "$exitmsg\n";
   trace("$exitmsg");
   if ($exitrc != 0)
   {
      trace("Display Mail List failed. displayMaillist.pl exiting with rc=" . $exitrc);
      error(MSG_DISPLAY_MAILMGR_FAILED, $exitrc);
   }
   else
   {
      audit(MSG_DISPLAY_MAILMGR_COMPLETED, 0);
   }
   exit $exitrc;
}

sub sig_handler
{
   my ($sig) = @_;
   audit(MSG_CAUGHT_SIGNAL, $sig);
   &cleanup_exit("Signal $sig caught.  Exiting..", 1);
}

sub setup_signals
{
   $SIG{'TERM'} = 'sig_handler';
   $SIG{'QUIT'} = 'sig_handler';
   $SIG{'INT'} = 'sig_handler';
   $SIG{'HUP'} = 'sig_handler';

   return( 0 );
}

&setup_signals();

$sendmail_mc = "/etc/sendmail.mc";
$enabled = 0;
openOrDie(IN, "<$sendmail_mc");
while (<IN>)
{
   if (/ALIAS_FILE/)
   {
      $enabled = 1;
   }
}
close(IN);

if ($enabled)
{
   print "ENABLED";
}
else
{
   print "DISABLED";
}
print " " . &get_admin_maillist();
&cleanup_exit("", 0);
