#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
#
# getImage.pl - get image and make it ready to install
#
# Usuage:
#   findImage.pl --ftpurl <ftp url address> [--help]
#
# Script called: /opt/CSM/scripts/xferFiles.pl
# 

use strict;

my ($eSP_BIN, $rc, $stdout, $stderr, $findImage_usage, @opts, $pkg_name, $pkg_name_file, $date);
my ($mounted, $ftpurl, $version, $release, $ptf, $file);

$eSP_BIN = "/opt/CSM/scripts";

use ASM::Common;
require "$eSP_BIN/common_install.pl";

setFacility('upgrade');

use vars qw/ $opt_help $opt_ftpurl $opt_img_type $opt_server $opt_img_dir $opt_subdir $MNT_PT /;

# Parse the arguments
@opts = ("img_type=s", "server=s", "img_dir=s", "img_subdir=s", "ftpurl=s", "help");
$findImage_usage = "\t\tor\nftpImage.pl --ftpurl=<url> ] ";

commonInit(\@opts, $findImage_usage);

$mounted = 0;

#
# First parameter is optional administrator name (-a admin name)
# If specified, all positional parameters are shifted by two
#

sub cleanup_exit
{
   my ($exitmsg, $exitrc) = @_;
 
   trace("$exitmsg");
   trace("exit $exitrc");

   if ($mounted)
   { 
      trace("Umounting $MNT_PT....");
      if (&umountit() != 0)
      {
         error(MSG_UNABLE_TO_UNMOUNT, $MNT_PT);
         trace("Unable to unmount $MNT_PT filesystem.");
      }
      else
      {
         trace("Completed unmount of $MNT_PT.");
      }
   }
   exit $exitrc;

}

sub sig_handler
{
   my ($sig) = @_;
   audit(MSG_CAUGHT_SIGNAL, $sig);
   &cleanup_exit("Signal $sig caught.   Exiting..", 1);
}

sub setup_signals
{
   $SIG{'TERM'} = 'sig_handler';
   $SIG{'QUIT'} = 'sig_handler';
   $SIG{'INT'} = 'sig_handler';
   $SIG{'HUP'} = 'sig_handler';

   return( 0 );
}

&setup_signals(); 

# Get command line options and check 'em out to see if they're okay.
 
&GetOptions(@opts);

if ($Getopt::Long::error)
{  # There was an error in parsing the arguments
   error(MSG_USAGE_ERROR, "findImage.pl", $findImage_usage);
   &cleanup_exit("Error:  Unknown problem with parsing arguments.", 2);
}

if($opt_help)
{
   &printUsage; 
}                                          

if ($opt_img_type eq "ftp")
{
   # Setup Command parameters
   $ftpurl = $opt_ftpurl;
   if (!$ftpurl)
   {
      error(MSG_NO_URL_SPECIFIED);
      &cleanup_exit("Error:  No URL specified.", 2);
   }

   if (!&isURLreachable($ftpurl))
   {
      error(MSG_UNABLE_TO_CONNECT, $ftpurl); 
      &cleanup_exit("Error:  Couldn't connect to server at URL: $ftpurl", 2);
   }
}
elsif ($opt_img_type eq "nfs")
{
   $rc = &mountit($opt_img_type, $opt_server, $opt_img_dir);
   if ($rc != 0)
   {
      error(MSG_UNABLE_TO_MOUNT, $opt_server . ":" .$opt_img_dir); 
      &cleanup_exit("Error:  Couldn't mount $opt_server:$opt_img_dir", $rc);
   }
   $mounted = 1;
   $ftpurl = $MNT_PT;
}
elsif ($opt_img_type eq "cd")
{
   $rc = &mountit($opt_img_type);
   if ($rc != 0)
   {
      error(MSG_UNABLE_TO_MOUNT, "CD"); 
      &cleanup_exit("Error:  Couldn't mount CD", $rc);
   }
   $mounted = 1;
   #  Remove leading and trailing slashes.
   chomp($opt_img_dir);
   $opt_img_dir =~ s/^\///;
   $opt_img_dir =~ s/\/$//;
   $ftpurl = "$MNT_PT/$opt_img_dir";
}
elsif ($opt_img_type eq "local")
{
   chomp($opt_img_dir);
   #  Remove trailing slashes.
   $opt_img_dir =~ s/\/$//;
   if ($opt_img_dir eq "")
   {
      error(MSG_COULD_NOT_OPEN_DIRECTORY, $opt_img_dir); 
      &cleanup_exit("Error:  Directory \"$opt_img_dir\" is invalid.", $rc);
   }
   $ftpurl = $opt_img_dir;
}
else
{
   &cleanup_exit("Must specify valid media type: cd, ftp, nfs, local filesystem", 1);
}



($version, $release, $ptf) = &findVersion();
trace("ASM Version: $version");
trace("ASM Release: $release");
trace("ASM PTF Level: $ptf");
#  Valid version, release, ptf
if (!($version =~ /^[0-9]*$/))
{
   error(MSG_INVALID_APPLIANCE_VERSION, $version);
   &cleanup_exit(getFormattedMessage(MSG_INVALID_APPLIANCE_VERSION, $version), -1);
}
if (!($release =~ /^[0-9]*$/))
{
   error(MSG_INVALID_APPLIANCE_RELEASE, $release);
   &cleanup_exit(getFormattedMessage(MSG_INVALID_APPLIANCE_RELEASE, $release), -1);
}

if (!($ptf =~ /^[0-9]*$/))
{
   error(MSG_INVALID_APPLIANCE_PTF_LEVEL, $ptf);
   &cleanup_exit(getFormattedMessage(MSG_INVALID_APPLIANCE_PTF_LEVEL, $ptf), -1);
}


foreach $file (&getPTFList($opt_img_type, $ftpurl, $version, $release, $ptf))
{
   print "$file\n";
   trace("Found PTF Level: $file");
}

&cleanup_exit("Completed search for PTFs.", 0);
