#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
#
# getImage.pl - get image and make it ready to install
#
# Usuage:
#   getImage.pl --img_type=<cd|local|ftp|nfs> [--svr_name=<server name>] [--img_dir=<Image Dir>] [--user_name=<User Name>] [--pasword=<Password>] [--pkgname=<Update Package Name>[--help]
#
# Script called: /opt/CSM/scripts/xferFiles.pl
# 
# Warning:
#   For "cd" and "nfs", the images are mounted on /mnt/cdrom 
#   and copied to /images/tmp. A regular directory on the image 
#   cannot contain any symbolic link. 
#                                          
# Wayne Gao, <waynegao$us.ibm.com> (29 Octobe 1999)
#

use strict;

my ($eSP_BIN, $rc, $stdout, $stderr, $getImage_usage, @opts, $MNT_PT, $file_list, $pkg_name, $pkg_name_file);
my ($svr_name, $img_type, $img_dir, $img_subdir, $user_name, $password, $date, $cmdopts, $logcmd);
my ($fn, $full_name, $file_out, @ftype, $status, $len, $dir_name);
my ($pkgupdate_name, $pkgupdate_filelist);
my ($PIDFILE, $PID, $pid, $STATUSFILE);

$eSP_BIN = "/opt/CSM/scripts";

use ASM::Common;
require "$eSP_BIN/common_install.pl";

setFacility('upgrade');

use vars qw/ $opt_help $opt_pkgname $opt_img_type $opt_svr_name $opt_img_dir $opt_img_subdir $opt_user_name $opt_password $opt_background $opt_status $opt_abort $IMG_DIR $IMG_TMP $STATUS_FILE/;

# Parse the arguments
@opts = ("pkgname=s", "img_type=s", "svr_name=s", "img_dir=s", "user_name=s", "password=s","help", "img_subdir=s", "status", "abort", "background");
$getImage_usage = "\t\tor\ngetImage.pl --img_type=<cd|local|nfs|ftp> [ --svr_name=<fq_server_name> ] [ --img_dir=<path to image dir on CD or remote server or NFS export directory> ] [ --img_subdir=<path under NFS export directory if any> ] [ --user_name=<name> ] [ --password=<name> ] [ --pkgname=<Update Package Name> ] [ --background ] [ --status ] [ --abort ] ";

commonInit(\@opts, $getImage_usage);

$MNT_PT = "/mnt/cdrom";
$file_list = "/tmp/file_list.txt";
$pkg_name_file = "/tmp/PKG_NAME";
$PIDFILE = "/images/copyimages.pid";

#
# First parameter is optional administrator name (-a admin name)
# If specified, all positional parameters are shifted by two
#

sub cleanup_exit
{
   my ($exitmsg, $exitrc) = @_;
   my ($rc);

   #  Cleanup PID file ONLY when its for the current background getImage command.
   #  Don't cleanup when its a status or abort getImage command.
   if (-f $PIDFILE)
   {
      my $PID = `cat $PIDFILE`;
      if ($$ == $PID)
      {
         $rc = runit([ "rm", "-f", $PIDFILE], \$stdout, \$stderr);
         if (($exitrc != 0) && ($exitrc != 77))
         {
            # Reset status file or indicate download of upgrade package failed.
            if ($pkg_name ne "")
            {
               $rc = runit(["echo pkgname=$pkg_name status=ERROR source=$img_type > $STATUS_FILE"], \$stdout, \$stderr);
            }
            else
            {
               $rc = runit(["> $STATUS_FILE"], \$stdout, \$stderr);
            }
         }
         if ($exitrc != 77)
         {
            #  Get rid of the filelist file
            if ($pkgupdate_name)
            {
               #  Create a filelist containing the update package files to transfer for the upgrade detection
               $pkgupdate_filelist = "/images/$pkgupdate_name.filelist";
               if (-f "$pkgupdate_filelist")
               {
                  $rc = runit(["rm -f $pkgupdate_filelist"], \$stdout, \$stderr);
               }
            }
         }
      }
   }

   print "$exitmsg\n";
   trace("$exitmsg");
   trace("exit $exitrc");
   $date = `date|tr -d "\n"`;
   trace("====== $date ======");
   exit $exitrc;

}

sub sig_handler
{
   my ($sig) = @_;
   audit(MSG_CAUGHT_SIGNAL, $sig);
   &cleanup_exit("Signal $sig caught.   Exiting..", 1);
}

sub setup_signals
{
   $SIG{'TERM'} = 'sig_handler';
   $SIG{'QUIT'} = 'sig_handler';
   $SIG{'INT'} = 'sig_handler';
   $SIG{'HUP'} = 'sig_handler';

   return( 0 );
}

sub sig_child
{
   my $pid = wait;

   my $rc = $? >> 8;
   trace("process $$ caught child process with rc ($rc)\n");
}

#
# Function: pid_status
# This function determines if a copy images operation is currently in progress (true).
#

sub pid_status
{
   if (-f $PIDFILE)
   {
      $PID = `cat $PIDFILE`;
      chomp($PID);
      $rc = runit(["kill", "-0", $PID], \$stdout, \$stderr);
      if ($rc == 0)
      {
         return (1);
      }
      else
      {
         return (0);
      }
   }
   else
   {
      return (0);
   }
}


#
# Function: pid_stop
# This function stops a copy images operation that is currently in progress.
#

sub pid_stop
{
   if (-f $PIDFILE)
   {
      $PID = `cat $PIDFILE`;
      chomp($PID);
      $rc = runit(["kill", "-SIGTERM", $PID], \$stdout, \$stderr);
      if ($rc == 0)
      {
         sleep(3);
         if (&pid_status)
         {
            $rc = runit(["kill", "-SIGKILL", $PID], \$stdout, \$stderr);
            if ($rc == 0)
            {
               sleep(3);
               if (&pid_status)
               {
                  #  Problem stopping Background getImage command.
                  return (0);
               }
               else
               {
               }
            }
            else
            {
               trace("Problem with kill -SIGKILL $PID command:  $stdout $stderr");
            }
         }
         #  Successfully stopped background getImage command.
         $rc = runit([ "rm", "-f", $PIDFILE], \$stdout, \$stderr);
         return (1);
      }
      else
      {
         trace("Problem with kill -SIGTERM $PID command:  $stdout $stderr");
      }
   }
   else
   {
      #  No background getImage is running so stop was successful anyway.
      return (1);
   }
   #  Problem stopping Background getImage command.
   return (0);
}

#
# Function: spawn
#
# Spawns (forks) the back or restore processes
#
sub spawn
{
   my ($rc);

   $rc = 0;

   FORK:
   {
      #  Always ignore child process.  Status is saved in getImage.status
      #$SIG{'CHLD'} = 'sig_child';
      $SIG{'CHLD'} = 'IGNORE';

      $rc = 77;
      if ($pid = fork)
      {
          trace "$pid spawned.\n";
          #sleep 10;
          #trace "$$ parent process done sleeping for 10 seconds.\n";
          trace "Current rc, $rc.\n";
      }
      elsif (defined $pid)
      {
         $SIG{'CHLD'} = 'IGNORE';
         `echo $$ > $PIDFILE`;
         #  Copying of images from CD, nfs, or ftp can now be done.
         &copy_images;
      }
      elsif ($! =~ /No more process/)
      {
         sleep 5;
         redo FORK;
      }
      else
      {
         trace("fork failed: $!");
         $rc = 2;
         cleanup_exit([MSG_CANNOT_START_NEW_PROCESS], $rc);
      }
   }
   return($rc);
}


&setup_signals(); 

# Get command line options and check 'em out to see if they're okay.
 
&GetOptions(@opts);

if ($Getopt::Long::error)
{  # There was an error in parsing the arguments
   error(MSG_USAGE_ERROR, "getImage.pl", $getImage_usage);
   &cleanup_exit("Error:  Unknown problem with parsing arguments.", 2);
}

if($opt_help)
{
   &printUsage; 
}                                          

#  Retrieve status of copying of images
if ($opt_status)
{
   &displaygetImageStatus();
   exit (0);
}

#  Cancel any running getImage.pl commands
#  Cancel previous image install requests by removing
#     all files in the images directories
if ($opt_abort)
{
   $rc = 0;

   if (&pid_status)
   {
      $rc = &pid_stop;
   }
   else
   {
      #  Remove images sitting in the images directories
      if ( -d "$IMG_DIR" )
      {
         trace("Directory $IMG_DIR exists, clean it up.");
         $rc = runit(["rm -rf $IMG_DIR/*"], \$stdout, \$stderr);
      }
      if ( -d "$IMG_TMP" )
      {
         trace("Directory $IMG_TMP exists, clean it up.");
         $rc = runit(["rm -rf $IMG_TMP/*"], \$stdout, \$stderr);
      }
      $rc = runit(["> $STATUS_FILE"], \$stdout, \$stderr);
   }
   exit ($rc);
}

if ($opt_img_type ne "cd" && $opt_img_type ne "nfs" && $opt_img_type ne "ftp" && $opt_img_type ne "local")
{
   trace("Error:  img_type is a required parameter with values: cd, local, nfs, or ftp.");
   print getFormattedMessage(MSG_IMG_TYPE_REQUIRED_ERROR) . "\n";
   error(MSG_USAGE_ERROR, "getImage.pl", $getImage_usage);
   printUsage;
}

if (&pid_status)
{
   cleanup_exit([MSG_BACKUP_RESTORE_IN_PROGRESS], 80);
}

# Setup Command parameters
$img_type = $opt_img_type;
$svr_name = $opt_svr_name;
$img_dir = $opt_img_dir;
$img_subdir = $opt_img_subdir;
$user_name = $opt_user_name;
$password = $opt_password;
$pkgupdate_name = $opt_pkgname;

$date = `date|tr -d "\n"`;
trace("====== Start of getImage.pl($date) ======");

trace("img_type=$img_type");
trace("svr_name=$svr_name");
trace("img_path=$img_dir");
trace("user_name=$user_name");
trace("img_dir=$img_dir");
trace("img_subdir=$img_subdir");

# Create the directories if necessary

if ( -d "$IMG_DIR" )
{
   trace("Directory $IMG_DIR exists, clean it up.");
   $rc = runit(["rm -rf $IMG_DIR/*"], \$stdout, \$stderr);
}
else
{
   trace("$IMG_DIR does not exist, creating $IMG_DIR");
   $rc = runit(["mkdir -p $IMG_DIR"], \$stdout, \$stderr);
   if ( $rc != 0 )
   {
      error(MSG_COULD_NOT_CREATE_DIRECTORY, $IMG_DIR);
      &cleanup_exit("ERROR: Cannot make $IMG_DIR.  rc = $rc.", 3);
   }
}
if ( -d "$IMG_TMP" )
{
   trace("Directory $IMG_TMP exists, clean it up.");
   $rc = runit(["rm -rf $IMG_TMP/*"], \$stdout, \$stderr);
}
else
{
   trace("$IMG_TMP does not exist, creating $IMG_TMP");
   $rc = runit(["mkdir -p $IMG_TMP"], \$stdout, \$stderr);
   if ($rc != 0 )
   {
      error(MSG_COULD_NOT_CREATE_DIRECTORY, $IMG_TMP);
      &cleanup_exit("ERROR: Cannot make $IMG_TMP.  rc = $rc", 3);
   }
}

#  Setup xfer options based on type of xfer. 
$cmdopts = "--img_type=$img_type --xfer=get --localdir=$IMG_TMP";
if ($img_type eq "nfs")
{
   $cmdopts = $cmdopts . " --svr_name=$svr_name --svr_dir=$img_dir";
   if ($img_subdir) {
     $cmdopts .= " --remotedir=$img_subdir";
   }
}
elsif ($img_type eq "ftp")
{
   $cmdopts = $cmdopts . " --svr_name=$svr_name --remotedir=$img_dir --user_name=$user_name --password=$password";
}
elsif ($img_type eq "local")
{
   if ($img_dir ne "")
   {
      $cmdopts = $cmdopts . " --remotedir=$img_dir";
   }
}
elsif ($img_type eq "cd")
{
   if ($img_dir ne "")
   {
      $cmdopts = $cmdopts . " --remotedir=$img_dir";
   }
}



#  Upgrade detection has all updates in one subdirectory.
if ($pkgupdate_name)
{
   #  Create a filelist containing the update package files to transfer for the upgrade detection
   #  Files to transfer:     <pkgupdate_name>.pkginfo.txt
   #                         <pkgupdate_name>.tar.gz
   $pkgupdate_filelist = "/images/$pkgupdate_name.filelist";
   $rc = runit(["echo \"$pkgupdate_name.pkgInfo.txt\n$pkgupdate_name.tar.gz\" > $pkgupdate_filelist"], \$stdout, \$stderr);
   $cmdopts = $cmdopts . " --filelist=$pkgupdate_filelist";
}

sub copy_images
{
   my ($rc);

   #
   #   Transfer Files from CD, NFS server, or FTP server
   #
   audit(MSG_TRANSFER_IMAGE_BEGAN, $img_type);
   #  Indicate in the getImage.pl status that this package is being downloaded.
   if ($pkgupdate_name)
   {
      $rc = runit(["echo pkgname=$pkgupdate_name status=DOWNLOADING source=$img_type > $STATUS_FILE"], \$stdout, \$stderr);
   }
   else
   {
      $rc = runit(["echo pkgname=UNKNOWN status=DOWNLOADING source=$img_type > $STATUS_FILE"], \$stdout, \$stderr);
   }

   trace("+++ Downloading images +++");
   trace("Transfering files. Wait....");
   $logcmd = $cmdopts . " ";
   $logcmd =~ s/--password=.* /--password=***** /;
   
   trace("Issuing command: perl $eSP_BIN/xferFiles.pl $logcmd");
   $rc = runit(["perl $eSP_BIN/xferFiles.pl $cmdopts"], \$stdout, \$stderr);
   #  Ignore signals (255) due to an issue with signals and background process.  
   if (($rc == 0) || ($rc == 255))
   {
      #  FTP doesn't maintain file permissions.
      chmod 0755, "$IMG_TMP/recover.sh", "$IMG_TMP/backup.sh", "$IMG_TMP/install.sh";
      audit(MSG_TRANSFER_IMAGE_COMPLETED, $img_type);
   }
   else
   {
      trace("Copy of software image (using $img_type) from server:directory $svr_name:$img_dir as user $user_name failed.");
      error(MSG_TRANSFER_IMAGE_FAILED);
      &cleanup_exit("Error: Failed to copy software image\n", 3);
   }
   
   trace("Checking transferred pkgInfo.txt at $IMG_TMP directory.");
   if ($pkgupdate_name)
   {
      #  Standardize the pkgInfo.txt from the update package.
      $rc = runit(["/bin/mv $IMG_TMP/$pkgupdate_name.pkgInfo.txt $IMG_TMP/pkgInfo.txt"], \$stdout, \$stderr);
   }
   if (-f "$IMG_TMP/pkgInfo.txt")
   {
      chop($pkg_name = `grep "PkgName" $IMG_TMP/pkgInfo.txt|grep -v "\#"|cut -f2 -d "="|tr -d " "`);
      trace("PkgName = $pkg_name");
      if ($pkg_name eq "")
      {
         error(MSG_TRANSFER_IMAGE_FAILED);
         &cleanup_exit( "ERROR: pkgInfo.txt exists but name cannot be extracted.\n", 3);
      }
   }
   else
   {
      error(MSG_TRANSFER_IMAGE_FAILED);
      &cleanup_exit("ERROR: pkgInfo.txt is not on $IMG_TMP.\n", 3);
   }
   
   trace("The package name is: $pkg_name.");
   $rc = runit(["echo $pkg_name > $pkg_name_file"], \$stdout, \$stderr);
   trace(`cat $pkg_name_file`);
   $rc = runit(["echo pkgname=$pkg_name status=DOWNLOADING source=$img_type  > $STATUS_FILE"], \$stdout, \$stderr);

   # We have mounted or ftp get images on /images/tmp 
   # Now let's uncompress and copy image to /images/install directory
   
   trace("List all files in /images/tmp");
   $rc = runit(["ls -1 $IMG_TMP > $file_list"], \$stdout, \$stderr);
   trace(`cat $file_list`);
   trace("++++ Uncompress, untar and move regular files to inst dir");
   trace("Uncompress, copying files to $IMG_DIR. Wait......");
   audit(MSG_PREPARE_IMAGE_BEGAN);
   openOrDie(LST, "< $file_list");
   while (<LST>)
   {
      chop($fn = $_);
      #  Filter out known troublesome characters in the rpm filenames.  There may be more....
      $fn =~ s/\+/\\\+/g;
      trace("++ Get file: $fn");
      $full_name = "$IMG_TMP/$fn";
      # Check if $full_name is a gziped file. If it is, unzip it.
      trace("Check if $full_name is a gziped file");
      $file_out = `file "$full_name"`;
      $rc = grep /gzip compressed data/, $file_out; 
      if ($rc > 0)
      {
         trace( "$full_name is a gnu gziped file (.gz)");
         trace("Unzip $full_name");
         $rc = runit(["gunzip $full_name"], \$stdout, \$stderr);
         #  Ignore signals (255) due to an issue with signals and background process.  
         if (($rc == 0) || ($rc == 255))
         {
            trace("Successfull unziped $full_name");
            trace("Extract name of the unziped file");
            $len = `echo $fn|wc -c` - 4;
            $fn = `echo $fn|cut -c 1-$len`;
            chomp($fn);
            trace("the unziped file is: $IMG_TMP/$fn");
            # After unzip, the .gz is removed from the file name. Replace 
            # $full_name by the new name.
            $full_name = "$IMG_TMP/$fn";
            $file_out = `file "$full_name"`;
         }
         else
         {
            error(MSG_PREPARE_IMAGE_FAILED);
            &cleanup_exit("Error: gunzip $full_name failed with rc=$rc", 3);
         } 
      }
      else
      {
         # Check if $full_name is a compressed file, uncompress it if it is.
         trace("Check if $full_name is a compressed file.");
         $rc = grep /compress\'d/, $file_out; 
         if ($rc > 0)
         {
            trace("$full_name is a compressed (.Z) file.");
            trace("Uncompress $full_name");
            $rc = runit(["uncompress $full_name"], \$stdout, \$stderr);
            #  Ignore signals (255) due to an issue with signals and background process.  
            if (($rc == 0) || ($rc == 255))
            {
               trace("Successfully uncompressed $full_name");
               trace("Extract name of the uncompressed file");
               $len = `echo $fn|wc -c` - 3;
               $fn = `echo $fn|cut -c 1-$len`;
               chomp($fn);
               trace("The uncompressed file is: $IMG_TMP/$fn");
               # After uncompress, the .Z is removed from the file name. Replace 
               # $full_name by the new full name.
               $full_name = "$IMG_TMP/$fn";
               $file_out = `file "$full_name"`;
           }
           else
           {
              error(MSG_PREPARE_IMAGE_FAILED);
              &cleanup_exit("ERROR: Uncompress $full_name failed rc=$rc\n",3);
           } 
         }
      }  
      #
      # At this point, if a file is compressed(ziped), the file has been 
      # uncompress with $full_name changed to the new uncompressed file.
      # Now we check if $full_name is a tar file. If it is, untar it. If
      # not, we assume the file is either a directory or a regular file.
      # Move it to $IMG_DIR.
      #
      # check $full_name is a tar file
      $rc = grep /tar archive/, $file_out; 
      if ($rc > 0)
      {
         trace("$full_name is a tar file. Untar and move to $IMG_DIR");
         trace("tar xvf $full_name");
         chdir $IMG_TMP;
         $rc = runit(["tar xvf $fn"], \$stdout, \$stderr);
         #  Ignore signals since it probably completed ok.
         if (($rc == 0) || ($rc == 255))
         {
            trace("Successfully untar $full_name");
         }
         else
         {
            error(MSG_PREPARE_IMAGE_FAILED);
            trace("TAR output: $stdout $stderr");
            &cleanup_exit("Error: untar $full_name failed.  rc=$rc", 3);
         }
         # Find the directory name associated with the tar file. That is to
         # trancate .tar from the file name.
         trace("Extract name of the directory");
         $len = `echo $fn|wc -c` - 5;
         $dir_name = `echo $fn|cut -c 1-$len`;
         chomp($dir_name);
         trace("The dir name is: $dir_name");
         trace("Move all files/dirs from $IMG_TMP/$dir_name to $IMG_DIR");
         trace("mv -f $IMG_TMP/$dir_name/* $IMG_DIR");
         $rc = runit(["/bin/mv -f $IMG_TMP/$dir_name/* $IMG_DIR"], \$stdout, \$stderr);
         #  Ignore signals since it probably completed ok.
         if (($rc == 0) || ($rc == 255))
         {
            trace("Successfully moved $IMG_TMP/$dir_name");
         }
         else
         {
            error(MSG_PREPARE_IMAGE_FAILED);
            &cleanup_exit("ERROR: Failed to move $IMG_TMP/$dir_name.\n", 3);
         }
      }
      else
      {
         trace("$full_name is a regular file or directory, move it.");
         $rc = runit(["/bin/mv -f $full_name $IMG_DIR"], \$stdout, \$stderr);
         #  Ignore signals since it probably completed ok.
         if (($rc == 0) || ($rc == 255))
         {
            trace("Successfully moved $full_name to $IMG_DIR");
         }
         else
         {
            error(MSG_PREPARE_IMAGE_FAILED);
            &cleanup_exit("Error: Failed ($rc) move $full_name to $IMG_DIR\nOutput of mv command: $stdout $stderr", 3);
        }
     }
   }
   trace("Clear the staging area.");
   trace("rm -rf $IMG_TMP/*");
   $rc = runit(["rm -rf $IMG_TMP/*"], \$stdout, \$stderr);
   
   audit(MSG_SOFTWARE_IMAGE_READY);
   $rc = runit(["echo pkgname=$pkg_name status=READY source=$img_type  > $STATUS_FILE"], \$stdout, \$stderr);
   &cleanup_exit("+++ Successfully obtained image +++", 0);
}

$rc = 0;
if ($opt_background)
{
   $rc = &spawn;
}
else
{
   &copy_images;
}

cleanup_exit("Download of image has been spawned.", $rc);
