#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
# installImage.pl - install an updgrade package during reboot
#
# Usage:  installImage.pl [--admin adminname]
#

use ASM::Common;

use strict;
my ($eSP_BIN, $rc, $stdout, $stderr, $installImage_usage, @opts, $pkg_name);

$eSP_BIN = "/opt/CSM/scripts";

require "$eSP_BIN/common_install.pl";

commonInit([],"");

setFacility('upgrade');

use vars qw/ $opt_help $IMG_DIR $STATUS_FILE /;

#  Parse the arguments  
@opts = ("admin=s", "help");
$installImage_usage = "\t\tor\ninstallImage.pl  [ --admin=name ]";

# Get command line options and check 'em out to see if they're okay.

&GetOptions(@opts);

if ($Getopt::Long::error){  # There was an error in parsing the arguments
  exit 2;
}

if($opt_help)
{
   &printUsage;
}

sub cleanup_exit
{
   my ($exitmsg, $exitrc) = @_;

   print "$exitmsg\n";
   trace("$exitmsg");
   if ($exitrc != 0)
   {
      trace("Upgrade failed. installImage.pl exiting with rc=" . $exitrc);
      error(MSG_SOFTWARE_UPGRADE_FAILED, $exitrc);
   }
   else
   {
      audit(MSG_SOFTWARE_UPGRADE_ACTIVATED, 0);
   }
   exit $exitrc;
}

sub sig_handler
{
   my ($sig) = @_;
   audit(MSG_CAUGHT_SIGNAL, $sig);
   &cleanup_exit("Signal $sig caught.  Exiting..", 1);
}

sub setup_signals
{
   $SIG{'TERM'} = 'sig_handler';
   $SIG{'QUIT'} = 'sig_handler';
   $SIG{'INT'} = 'sig_handler';
   $SIG{'HUP'} = 'sig_handler';

   return( 0 );
}

&setup_signals();

if (-f "$IMG_DIR/pkgInfo.txt")
{
   chop($pkg_name = `grep "PkgName" $IMG_DIR/pkgInfo.txt|grep -v "\#"|cut -f2 -d "="|tr -d " "`);
   trace("PkgName = $pkg_name");
   if ($pkg_name eq "")
   {
      error(MSG_TRANSFER_IMAGE_FAILED);
      &cleanup_exit( "ERROR: pkgInfo.txt exists but name cannot be extracted.\n", 3);
   }
}
else
{
   error(MSG_TRANSFER_IMAGE_FAILED);
   &cleanup_exit("ERROR: pkgInfo.txt is not on $IMG_DIR.\n", 3);
}
   
trace("The package name is: $pkg_name.");

# Create file to flag the update as ready to install.
# During system boot, the update image will be installed.
if ( -e "$IMG_DIR")
{
   trace("++++ Creating $IMG_DIR/INSTALL.READY file +++++");
   $rc = runit(["/bin/touch $IMG_DIR/INSTALL.READY"], \$stdout, \$stderr);
   if ($rc != 0)
   {
      &cleanup_exit("Unable to create $IMG_DIR/INSTALL.READY file.", 3);
   }
   $rc = runit(["echo pkgname=$pkg_name status=INSTALL source= > $STATUS_FILE"], \$stdout, \$stderr);
}
else
{
   &cleanup_exit("ERROR: Dir $IMG_DIR does not exist.", 3);
}

&cleanup_exit("Software Upgrade will occur on next system reboot.", 0);
