#!/usr/bin/perl -Uw
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# setEmailAlias.pl
#
# Sets an alias for an email user
#
# Usage:
#   setEmailAlias.pl --from=address --to=another_address
#           Alias so email to 'address' goes to 'another_address'
#   setEmailAlias.pl --delete --from=address
#           Remove alias for 'address'

use ASM::Exec;
use ASM::Common;
commonInit(["to=s", "from=s", "delete"],'--from=xxx [--to=yyy | --delete]');

if ((!$opt_to  && !$opt_delete) || !$opt_from) {
  printUsage();
}

setFacility('sysuser');
openOrDie(IN, "</etc/aliases");
my @aliases;
my $from_seen = 0;
while (<IN>) {
  if (/^\s*$opt_from:/) {
    $from_seen = 1;
    push(@aliases, "$opt_from: $opt_to\n") unless $opt_delete;
  } else {
    push(@aliases, $_);
  }
}
if (!$from_seen && !$opt_delete) {
  push(@aliases, "$opt_from: $opt_to\n");
}
close(IN);
openOrDie(OUT, ">/etc/aliases");
print OUT @aliases;
close(OUT);

# Make new aliases take effect
$rc = runit(["/usr/bin/newaliases"], \$stdout, \$stderr);
if ($rc != 0) {
  error(MSG_SENDMAIL_UPDATE_ERROR);
  trace("Errors running newaliases: $stdout $stderr\n");
  exit($rc);
}

if ($opt_delete) {
  audit(MSG_REMOVED_EMAIL_ALIAS, $opt_from);
} else {
  audit(MSG_SET_EMAIL_ALIAS, $opt_from, $opt_to); 
}
