#!/usr/bin/perl -U -w
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#


# setupNosuid.pl modifies the /etc/fstab file to turn off setuid for the 
# /home partition
#
# Pierre Carlson <mpcarl@us.ibm.com> (19 October 1999)
# Dan Poirier <poirier@us.ibm.com> (27 September 2000)
#
# This script will be called at install time by RPM inorder to 
# modify the the /etc/fstab file.  This will turn off suid support to the
# partation which contains /home.
# If /home2 exists it will set it there also.

# Modules to use
use Fcntl;
use DB_File;

use ASM::Common;
commonInit([],"");

$fstabFile = "/etc/fstab";

#open the file
if(!-e $fstabFile) {
  trace("file $fstabFile does not exist");
  error(MSG_COULD_NOT_OPEN_FILE, $fstabFile);
#    print LOG "$fstabFile does not exist\n";
    exit 3;
}

openOrDie(FSTAB, "+< $fstabFile");
@ARRAY = <FSTAB>;

# get the line numnbers of the entries we care about
$i=0; # counter
$homeLine=-1;
$home2Line=-1;
$slashLine=-1;
while (defined $ARRAY[$i]) {
    # find /home
    if ($ARRAY[$i] =~ /\/home /) {	
	$homeLine = $i;
    } # end if
    if ($ARRAY[$i] =~ /\/home2 /) {	
	$home2Line = $i;
    } # end if
    if ($ARRAY[$i] =~ / \/ /) {	
	$slashLine = $i;
    } # end if
    $i++;
} # end while

if ($homeLine != -1) {
    &configPart("/home", $homeLine);
} # end /home
# Don't mark / as nosuid!
#elsif ($slashLine != -1) { # not a /home directory, so use /
#    &configPart("/", $slashLine);
#}
else { # Should not get here
  trace("could not find /home in $fstabFile");
  error(MSG_UNEXPECTED_ERROR);
  close(FSTAB)                 or die "Closing: $!";
  exit 3;
}

# Since / or /home worked, go ahead and
# do /home2 if it exists
if ($home2Line != -1) {
    &configPart("/home2", $home2Line);
} # end /home2

# write back the array and close file
seek(FSTAB,0,0)              or do {
  trace("error seeking in $fstabFile: $!");
  error(MSG_ERROR_WRITING_FILE, $fstabFile);
  exit(3);
};
print FSTAB @ARRAY           or do {
  trace "Printing: $!";
  error(MSG_ERROR_WRITING_FILE, $fstabFile);
  exit(3);
};
truncate(FSTAB,tell(FSTAB))     or do {
  trace "Truncating: $!";
  error(MSG_ERROR_WRITING_FILE, $fstabFile);
  exit(3);
};
close(FSTAB)                 or do {
  trace "Closing: $!";
  error(MSG_ERROR_WRITING_FILE, $fstabFile);
  exit(3);
};
exit 0;

# sub to configure partition for quota
# takes two parameters partition
# array subscript to modify
sub configPart {
    my $part = shift(@_);
    my $line = shift(@_);
    # modify the fstab file
    # make sure that the entry is not already there
    if ($ARRAY[$line] !~ /nosuid/) {		
	@temp = split(/\s+/, $ARRAY[$line]);
        $ARRAY[$line] =~ s/(\s+)$temp[3](\s+)/$1$temp[3],nosuid$2/;
    } # end if
}
