#!/usr/bin/perl -U -w
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#


# setupQuota.pl modifies the /etc/fstab file to turn on quota for the 
# /home partition
#
# Pierre Carlson <mpcarl@us.ibm.com> (19 October 1999)
#
# This script will be called at install time by RPM inorder to 
# modify the the /etc/fstab file.  This will add quota support to the
# partation which contains /home.  This might be /home or just slash.
# If also creates the quota.user file in the correct location.
# If /home2 exists it will set it there also.

# Modules to use
use Fcntl;
use DB_File;
use ASM::Exec;

use ASM::Common;
commonInit([],"");


$fstabFile = "/etc/fstab";

#open the file
if(!-e $fstabFile) {
  trace("file $fstabFile does not exist");
  error(MSG_COULD_NOT_OPEN_FILE, $fstabFile);
#    print LOG "$fstabFile does not exist\n";
    exit 3;
}

openOrDie(FSTAB, "+< $fstabFile");
@ARRAY = <FSTAB>;

# get the line numnbers of the entries we care about
$i=0; # counter
$homeLine=-1;
$home2Line=-1;
$slashLine=-1;
while (defined $ARRAY[$i]) {
    # find /home
    if ($ARRAY[$i] =~ /\/home /) {	
	$homeLine = $i;
    } # end if
    if ($ARRAY[$i] =~ /\/home2 /) {	
	$home2Line = $i;
    } # end if
    if ($ARRAY[$i] =~ / \/ /) {	
	$slashLine = $i;
    } # end if
    $i++;
} # end while

if ($homeLine != -1) {
    &configPart("/home", $homeLine);
} # end /home
elsif ($slashLine != -1) { # not a /home directory, so use /
    &configPart("/", $slashLine);
}
else { # Should not get here
    print STDERR "setupQuota.pl could not find / or /home";
    close(FSTAB)                 or die "Closing: $!";
    exit 3;
}

# Since / or /home worked, go ahead and
# do /home2 if it exists
if ($home2Line != -1) {
    &configPart("/home2", $home2Line);
} # end /home2

# write back the array and close file
seek(FSTAB,0,0)              or do {
  trace("error seeking in $fstabFile: $!");
  error(MSG_ERROR_WRITING_FILE, $fstabFile);
  exit(3);
};
print FSTAB @ARRAY           or do {
  trace "Printing: $!";
  error(MSG_ERROR_WRITING_FILE, $fstabFile);
  exit(3);
};
truncate(FSTAB,tell(FSTAB))     or do {
  trace "Truncating: $!";
  error(MSG_ERROR_WRITING_FILE, $fstabFile);
  exit(3);
};
close(FSTAB)                 or do {
  trace "Closing: $!";
  error(MSG_ERROR_WRITING_FILE, $fstabFile);
  exit(3);
};

# This used to force a quotacheck every boot, but it
# makes a reboot extremely slow!!!
# We already run quotacheck on any partition when we enable quotas
# (see below), and /etc/rc.d/rc.sysinit will run it again if fsck
# reports that it made any changes to the file system, so that should
# cover us.
#&updateRC ();

exit 0;

# sub to configure partition for quota
# takes two parameters partition
# array subscript to modify
sub configPart {
    my $part = shift(@_);
    my $line = shift(@_);
    my $quotaFile = "";
    if ($part eq "/") {
	    $quotaFile = "/quota.user";
	}
    else {
	$quotaFile = "$part/quota.user";
    }
    # modify the fstab file
    # make sure that the entry is not already there
    if ($ARRAY[$line] !~ /usrquota/) {		
	@temp = split(/\s+/, $ARRAY[$line]);
        $ARRAY[$line] =~ s/(\s+)$temp[3](\s+)/$1$temp[3],usrquota$2/;
    } # end if
    # create the needed file and set the permissions on it.
    if (!-e $quotaFile) { # if file does not already exist
	#printf LOG "Creating $quotaFile\n";
	#system("touch $quotaFile");
	#system("chown root:nobody $quotaFile");
	#system("chmod 660 $quotaFile");
        my ($rc,$stdout,$stderr);
        $rc = runit(["/sbin/quotacheck", $part], \$stdout, \$stderr);
	if ($rc != 0) {
	  trace("error running quotacheck: rc=$rc\n$stdout\n$stderr");
	  error(MSG_UNEXPECTED_ERROR);
	  exit 3;
	}
    }
}

#sub routine to update rc.sysinit
# sub updateRC {
    
#     my($rcFile)="/etc/rc.d/rc.sysinit";
#     my($alreadyInstalled)=0;

#     $H=tie(@h,'DB_File',$rcFile,O_RDWR,0644,$DB_RECNO)
# 	or die "can't update /tmp/sampe: $!";

#     for ($i=0; $i<$H->length; ++$i) {
# 	$_=$h[$i];
# 	if(/IBM - Check Quotas/i){
# 	    $alreadyInstalled=1; 
# 	    return ;
# 	}
# 	if(/Turning on user and group quotas for local filesystems/i){
# 	    if($alreadyInstalled==0) {
# 	      # Leave this in English, it is not seen normally
# 		$h[$i-2]= "\nif [ -x /sbin/quotacheck ]; then # IBM - Check Quotas\n echo \"Checking quotas.  This may take some time.\" \n /sbin/quotacheck -a \n fi\n";
# 	    }
# 	}
#     }
#     untie @H;
# } # end sub updateRC
