#!/usr/bin/perl
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# unconfigureMailmgr
#
# Uonfigures/Deactivates Mail List Mgr

use ASM::Exec;
use ASM::Common;

my ($eSP_BIN, $rc, $stdout, $stderr, $wrapper);
$eSP_BIN = "/opt/CSM/scripts";

require "$eSP_BIN/common_mailmgr.pl";

use strict;

commonInit([],'');

setFacility('sysuser');

use vars qw/ $MAILMGR_ALIASES $SECURE_WRAPPER /;

my $sendmail_cf = "/etc/sendmail.cf";
my $sendmail_mc = "/etc/sendmail.mc";
my $mjaliases = "$MAILMGR_ALIASES";

  # Start with /etc/sendmail.mc
  # If no smarthost and we have one, then add it

my @config;
push(@config, "# Sendmail configuration created by $0.\n");

openOrDie(IN, "<$sendmail_mc");
while (<IN>)
{
   if (/ALIAS_FILE/)
   {
      next;			# skip ALIAS_FILE entry which should be the Mail List one
   }
   push(@config, $_);
}
close(IN);

openOrDie(OUT, ">$sendmail_mc");
print OUT @config;
close(OUT);

# "Compile" it
$rc = runit(["/usr/bin/m4", $sendmail_mc], \$stdout, \$stderr);
if ($rc != 0) {
  error(MSG_SENDMAIL_UPDATE_ERROR);
  trace("Errors from m4: $stderr\n");
  exit(1);
}

# Backup config if not previously done (at least for development
# so I don't have to completely re-install to fix myself...)
if (! -f "${sendmail_cf}-backup") {
  system("cp $sendmail_cf ${sendmail_cf}-backup");
}

# Replace config
openOrDie(OUT, ">$sendmail_cf");
print OUT $stdout;
close(OUT);

# Restart sendmail
$rc = runit(["/etc/rc.d/init.d/sendmail", "restart"], \$stdout, \$stderr);
if ($rc != 0) {
  error(MSG_SENDMAIL_UPDATE_ERROR);
  trace("Errors restarting sendmail: $stdout $stderr\n");
}

$wrapper = "$SECURE_WRAPPER";

$rc = runit(["/bin/rm -f $wrapper"], \$stdout, \$stderr);
if ($rc != 0) {
  error(MSG_UNCONFIG_MAILMGR_FAILED);
  trace("Error removing wrapper $wrapper from sendmail: $stdout $stderr\n");
  exit($rc);
}

audit(MSG_UNCONFIG_MAILMGR_COMPLETED);
exit(0);
